unit REOExamp;
// TRichEditOLE example program.
// Copyright  1997 by David R. Michael.
// All Rights Reserved.
//
// This example program is "missing" all the bell and whistles usually
// accompanying Rich Text example programs. I did this on purpose. I am
// demonstrating the OLE capabilities of TRichEditOle. I assume you are
// already familiar with the formatting features of TRichEdit. If not, the
// TRichEdit example program that comes with Delphi 3 is an excellant
// source of information.
//
// TRichEditOLE can:
// - Use the Windows 95 "Insert Object..." dialog
// - Use the Windows 95 "Paste Special..." dialog
// - Accept OLE drag-and-drop operations
// - Fill a menu with the "verbs" available to an object

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, REOCtls, Menus, RichEdit;

type
  TREOExampleForm = class(TForm)
    MainMenu1: TMainMenu;
    PopupMenu1: TPopupMenu;
    RichEditOle1: TRichEditOle;
    File1: TMenuItem;
    Edit1: TMenuItem;
    Help1: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Paste1: TMenuItem;
    PasteSpecial1: TMenuItem;
    InsertObject1: TMenuItem;
    About1: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Undo1: TMenuItem;
    Cut2: TMenuItem;
    Copy2: TMenuItem;
    Paste2: TMenuItem;
    PasteSpecial2: TMenuItem;
    Object1: TMenuItem;
    N2: TMenuItem;
    InsertObject2: TMenuItem;
    N3: TMenuItem;
    N4: TMenuItem;
    procedure New1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Undo1Click(Sender: TObject);
    procedure Edit1Click(Sender: TObject);
    procedure Cut1Click(Sender: TObject);
    procedure Copy1Click(Sender: TObject);
    procedure Paste1Click(Sender: TObject);
    procedure PasteSpecial1Click(Sender: TObject);
    procedure InsertObject1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure About1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
  private
    { Private declarations }
    FFileName: string;
    FModified: boolean;
    procedure SetFileName(value:string);
    procedure SetModified(value:boolean);
    procedure OpenFile(aFileName:string);
    procedure CheckFileSave;
    procedure SetEditRect;
  public
    { Public declarations }
    property FileName: string read FFileName write SetFileName;
    property Modified: boolean read FModified write SetModified;
  end;

var
  REOExampleForm: TREOExampleForm;

implementation

uses REOAbout;

{$R *.DFM}

procedure TREOExampleForm.SetFileName(value:string);
   begin
   FFileName:=value;
   Caption:=Format('%s - %s',[ExtractFileName(FFileName),Application.Title]);
   end;

procedure TREOExampleForm.SetModified(value:boolean);
   begin
   FModified:=value;
   end;

procedure TREOExampleForm.OpenFile(aFileName:string);
   begin
   RichEditOle1.Lines.LoadFromFile(aFileName);
   FileName:=aFileName;
   RichEditOle1.SetFocus;
   RichEditOle1.Modified:=false;
   Modified:=false;
   end;

procedure TREOExampleForm.CheckFileSave;
   var
      saveResp: Integer;
   begin
   if not RichEditOle1.Modified then Exit;
   saveResp:=MessageDlg(Format('Save changes to %s?',[FFileName]),mtConfirmation,mbYesNoCancel,0);
   case saveResp of
      idYes: Save1Click(Self);
      idNo: {Nothing};
      idCancel: Abort;
      end;
   end;

procedure TREOExampleForm.SetEditRect;
   var
      R: TRect;
   begin
   with RichEditOle1 do
      begin
      R:=Rect(10,0,ClientWidth-10,ClientHeight);
      SendMessage(Handle,EM_SETRECT,0,longint(@R));
      end;
   end;

// Event Handlers
procedure TREOExampleForm.New1Click(Sender: TObject);
   begin
   FileName:='Untitled';
   RichEditOle1.SelectAll;
   RichEditOle1.ClearSelection;
   RichEditOle1.Modified:=false;
   Modified:=false;
   end;

procedure TREOExampleForm.Open1Click(Sender: TObject);
   begin
   CheckFileSave;
   if OpenDialog1.Execute then
      OpenFile(OpenDialog1.FileName);
   end;

procedure TREOExampleForm.Save1Click(Sender: TObject);
   begin
   if FileName='Untitled' then
      SaveAs1Click(Sender)
   else
      begin
      RichEditOle1.Lines.SaveToFile(FileName);
      RichEditOle1.Modified:=false;
      Modified:=false;
      end;
   end;

procedure TREOExampleForm.SaveAs1Click(Sender: TObject);
   begin
   if SaveDialog1.Execute then
      begin
      if FileExists(SaveDialog1.FileName) then
         if MessageDlg(Format('OK to overwrite %s',[SaveDialog1.FileName]),
               mtConfirmation,mbYesNoCancel,0)<>idYes then Exit;
      RichEditOle1.Lines.SaveToFile(SaveDialog1.FileName);
      FileName:=SaveDialog1.FileName;
      RichEditOle1.Modified:=false;
      Modified:=false;
      end;
   end;

procedure TREOExampleForm.FormResize(Sender: TObject);
   begin
   SetEditRect;
   end;

procedure TREOExampleForm.FormPaint(Sender: TObject);
   begin
   SetEditRect;
   end;

procedure TREOExampleForm.Undo1Click(Sender: TObject);
   begin
   SendMessage(RichEditOle1.Handle,EM_UNDO,0,0);
   end;

procedure TREOExampleForm.Edit1Click(Sender: TObject);
   begin
   Cut1.Enabled:=RichEditOle1.SelLength>0;
   Copy1.Enabled:=RichEditOle1.SelLength>0;
   Undo1.Enabled:=Boolean(SendMessage(RichEditOle1.Handle,EM_CANUNDO,0,0));
   Paste1.Enabled:=Boolean(SendMessage(RichEditOle1.Handle,EM_CANPASTE,0,0));
   PasteSpecial1.Enabled:=Boolean(SendMessage(RichEditOle1.Handle,EM_CANPASTE,0,0));
   end;

procedure TREOExampleForm.Cut1Click(Sender: TObject);
   begin
   RichEditOle1.CutToClipboard;
   end;

procedure TREOExampleForm.Copy1Click(Sender: TObject);
   begin
   RichEditOle1.CopyToClipboard;
   end;

procedure TREOExampleForm.Paste1Click(Sender: TObject);
   begin
   RichEditOle1.PasteFromClipboard;
   end;

procedure TREOExampleForm.PasteSpecial1Click(Sender: TObject);
   begin
   RichEditOle1.PasteSpecialFromClipboard;
   end;

procedure TREOExampleForm.InsertObject1Click(Sender: TObject);
   begin
   RichEditOle1.InsertObject;
   end;

procedure TREOExampleForm.FormShow(Sender: TObject);
   begin
   FileName:='Untitled';
   end;

procedure TREOExampleForm.PopupMenu1Popup(Sender: TObject);
   begin
   Cut2.Enabled:=RichEditOle1.SelLength>0;
   Copy2.Enabled:=RichEditOle1.SelLength>0;
   Paste2.Enabled:=Boolean(SendMessage(RichEditOle1.Handle,EM_CANPASTE,0,0));
   PasteSpecial2.Enabled:=Boolean(SendMessage(RichEditOle1.Handle,EM_CANPASTE,0,0));
   Object1.Enabled:=(RichEditOle1.OleObject<>nil);
   RichEditOle1.FillPopup(Object1);
   end;

procedure TREOExampleForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
   begin
   CheckFileSave;
   end;

procedure TREOExampleForm.About1Click(Sender: TObject);
   begin
   AboutBox.ShowModal;
   end;

procedure TREOExampleForm.Exit1Click(Sender: TObject);
   begin
   Close;
   end;

end.
